unit FreeOTFECDBDump;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, ComCtrls,
  PasswordRichEdit, Spin64,
  OTFEFreeOTFE_U, Buttons;

type
  TfrmFreeOTFECDBDump = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    edVolumeFilename: TEdit;
    Label2: TLabel;
    Label9: TLabel;
    se64Offset: TSpinEdit64;
    Label10: TLabel;
    preUserKey: TPasswordRichEdit;
    seSaltLength: TSpinEdit;
    Label7: TLabel;
    Label5: TLabel;
    OpenDialog: TOpenDialog;
    GroupBox2: TGroupBox;
    pbBrowseDumpfile: TButton;
    edDumpFilename: TEdit;
    Label3: TLabel;
    SaveDialog: TSaveDialog;                  
    bbBrowsePartition: TBitBtn;
    bbBrowseFile: TBitBtn;
    seKeyIterations: TSpinEdit;
    Label4: TLabel;
    procedure pbBrowseDumpfileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ControlChanged(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure bbBrowseFileClick(Sender: TObject);
    procedure bbBrowsePartitionClick(Sender: TObject);
  private
    function GetVolumeFilename(): string;
    function GetUserKey(): string;
    function GetOffset(): int64;
    function GetSaltLength(): integer;
    function GetKeyIterations(): integer;
    function GetDumpFilename(): string;

    procedure EnableDisableControls();
  public
    OTFEFreeOTFE: TOTFEFreeOTFE;

    property VolumeFilename: string read GetVolumeFilename;
    property UserKey: string read GetUserKey;
    property Offset: int64 read GetOffset;
    property SaltLength: integer read GetSaltLength;
    property KeyIterations: integer read GetKeyIterations;
    property DumpFilename: string read GetDumpFilename;
  end;


implementation

{$R *.DFM}

uses
  SDUGeneral;

const
  CRLF = #10+#13;

procedure TfrmFreeOTFECDBDump.pbBrowseDumpfileClick(Sender: TObject);
begin
  SDUOpenSaveDialogSetup(SaveDialog, edDumpFilename.text);

  if SaveDialog.Execute() then
    begin
    edDumpFilename.text := SaveDialog.Filename;
    end;

end;


function TfrmFreeOTFECDBDump.GetVolumeFilename(): string;
begin
  Result := edVolumeFilename.text;
end;

function TfrmFreeOTFECDBDump.GetUserKey(): string;
begin
  Result := preUserKey.Text;
end;

function TfrmFreeOTFECDBDump.GetOffset(): int64;
begin
  Result := se64Offset.Value;
end;

function TfrmFreeOTFECDBDump.GetSaltLength(): integer;
begin
  Result := seSaltLength.Value;
end;

function TfrmFreeOTFECDBDump.GetKeyIterations(): integer;
begin
  Result := seKeyIterations.Value;
end;

function TfrmFreeOTFECDBDump.GetDumpFilename(): string;
begin
  Result := edDumpFilename.text;
end;


procedure TfrmFreeOTFECDBDump.FormCreate(Sender: TObject);
begin
  preUserKey.Plaintext := TRUE;
  // FreeOTFE volumes CAN have newlines in the user's password
  preUserKey.WantReturns := TRUE;
  preUserKey.WordWrap := TRUE;
  preUserKey.Lines.Clear();
  preUserKey.PasswordChar := '*';

  seSaltLength.Increment := 8;
  seSaltLength.Value := DEFAULT_SALT_LENGTH;

  se64Offset.Value := 0;

  seKeyIterations.MinValue := 1;
  seKeyIterations.MaxValue := 999999; // Need *some* upper value, otherwise setting MinValue won't work properly
  seKeyIterations.Increment := DEFAULT_KEY_ITERATIONS_INCREMENT;
  seKeyIterations.Value := DEFAULT_KEY_ITERATIONS;

  edVolumeFilename.text := '';
  edDumpFilename.text := '';

end;

procedure TfrmFreeOTFECDBDump.EnableDisableControls();
begin
  pbOK.Enabled := (
                   (edVolumeFilename.text <> '') AND
                   (edDumpFilename.text <> '') AND
                   (KeyIterations > 0)
                  );
end;

procedure TfrmFreeOTFECDBDump.FormShow(Sender: TObject);
begin
  EnableDisableControls();

end;

procedure TfrmFreeOTFECDBDump.ControlChanged(Sender: TObject);
begin
  EnableDisableControls()
end;

procedure TfrmFreeOTFECDBDump.pbOKClick(Sender: TObject);
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
var
  startTime: TDateTime;
  stopTime: TDateTime;
  diffTime: TDateTime;
  Hour, Min, Sec, MSec: Word;
{$ENDIF}
begin
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
  startTime := Now();
{$ENDIF}
  if OTFEFreeOTFE.DumpCriticalDataToFile(
                           VolumeFilename,
                           Offset,
                           UserKey,
                           SaltLength,  // In bits
                           KeyIterations,
                           DumpFilename
                          ) then
    begin
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
    stopTime := Now();
    diffTime := (stopTime - startTime);
    DecodeTime(diffTime, Hour, Min, Sec, MSec);
    showmessage('Time taken to dump CDB: '+inttostr(Hour)+' hours, '+inttostr(Min)+' mins, '+inttostr(Sec)+'.'+inttostr(MSec)+' secs');
{$ENDIF}

    MessageDlg('A human readable copy of your critical data block has been '+CRLF+
               'written to:'+CRLF+
               CRLF+
               DumpFilename,
               mtInformation, [mbOK], 0);

    ModalResult := mrOK;
    end
  else
    begin
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
    stopTime := Now();
    diffTime := (stopTime - startTime);
    DecodeTime(diffTime, Hour, Min, Sec, MSec);
    showmessage('Time taken to FAIL to dump CDB: '+inttostr(Hour)+' hours, '+inttostr(Min)+' mins, '+inttostr(Sec)+'.'+inttostr(MSec)+' secs');
{$ENDIF}

    MessageDlg('Unable to dump out critical data block.'+CRLF+
               CRLF+
               'Please ensure that your password and details are entered '+CRLF+
               'correctly, and that this file is not currently in use (e.g. mounted)',
               mtError, [mbOK], 0);
    end;

end;


procedure TfrmFreeOTFECDBDump.bbBrowseFileClick(Sender: TObject);
begin
  SDUOpenSaveDialogSetup(OpenDialog, edVolumeFilename.text);

  if OpenDialog.Execute() then
    begin
    edVolumeFilename.text := OpenDialog.Filename;
    edDumpFilename.text := edVolumeFilename.text + '.txt';
    end;

end;

procedure TfrmFreeOTFECDBDump.bbBrowsePartitionClick(Sender: TObject);
var
  selectedPartition: string;
begin
  selectedPartition := OTFEFreeOTFE.SelectPartition();
  if (selectedPartition <> '') then
    begin
    edVolumeFilename.text := selectedPartition;
    end;

end;

END.


