unit FreeOTFEMain;
// Description: 
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Spin, StdCtrls, ComCtrls, ImgList, ExtCtrls, Grids, Menus, OTFE_U,
  OTFEFreeOTFE_U, Shredder, MouseRNGDialog_U;

type
  TfrmFreeOTFEMain = class(TForm)
    ilDriveIcons: TImageList;
    lvDrives: TListView;
    pmDrives: TPopupMenu;
    miPopupDismount: TMenuItem;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    miFreeOTFEMountFile: TMenuItem;
    miDismountMain: TMenuItem;
    miExit: TMenuItem;
    N2: TMenuItem;
    miPopupProperties: TMenuItem;
    N3: TMenuItem;
    miFreeOTFENew: TMenuItem;
    View1: TMenuItem;
    miProperties: TMenuItem;
    N1: TMenuItem;
    miRefresh: TMenuItem;
    OpenDialog: TOpenDialog;
    Help1: TMenuItem;
    About1: TMenuItem;
    miDismountAllMain: TMenuItem;
    miPopupDismountAll: TMenuItem;
    N4: TMenuItem;
    miLinuxVolume: TMenuItem;
    miLinuxNew: TMenuItem;
    miLinuxMountFile: TMenuItem;
    OTFEFreeOTFE: TOTFEFreeOTFE;
    N5: TMenuItem;
    miFreeOTFEDrivers: TMenuItem;
    miLinuxDismount: TMenuItem;
    miTools: TMenuItem;
    miOverwriteFreeSpace: TMenuItem;
    miFormat: TMenuItem;
    N6: TMenuItem;
    miPopupFormat: TMenuItem;
    miPopupOverwriteFreeSpace: TMenuItem;
    MouseRNGDialog1: TMouseRNGDialog;
    N7: TMenuItem;
    miPortableModeDrivers: TMenuItem;
    N8: TMenuItem;
    miCDBBackup: TMenuItem;
    miCDBRestore: TMenuItem;
    miCreateKeyfile: TMenuItem;
    miChangePassword: TMenuItem;
    miCDBPlaintextDump: TMenuItem;
    miCDB: TMenuItem;
    miFreeOTFEMountPartition: TMenuItem;
    miLinuxMountPartition: TMenuItem;
    procedure miExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure miRefreshClick(Sender: TObject);
    procedure miNewClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure lvDrivesResize(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure miFreeOTFEMountFileClick(Sender: TObject);
    procedure miPropertiesClick(Sender: TObject);
    procedure miPopupPropertiesClick(Sender: TObject);
    procedure miDismountClick(Sender: TObject);
    procedure miDismountAllClick(Sender: TObject);
    procedure lvDrivesClick(Sender: TObject);
    procedure lvDrivesDblClick(Sender: TObject);
    procedure lvDrivesChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure miLinuxMountFileClick(Sender: TObject);
    procedure miLinuxNewClick(Sender: TObject);
    procedure miFreeOTFEDriversClick(Sender: TObject);
    procedure miOverwriteFreeSpaceClick(Sender: TObject);
    procedure miFormatClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure miPortableModeDriversClick(Sender: TObject);
    procedure miCDBBackupClick(Sender: TObject);
    procedure miCDBRestoreClick(Sender: TObject);
    procedure miCreateKeyfileClick(Sender: TObject);
    procedure miChangePasswordClick(Sender: TObject);
    procedure miCDBPlaintextDumpClick(Sender: TObject);
    procedure miFreeOTFEMountPartitionClick(Sender: TObject);
    procedure miLinuxMountPartitionClick(Sender: TObject);

  private
    // Flag to indicate that the executable is about to exit
    ShuttingDownFlag: boolean;

    TempCypherDriver: string;
    TempCypherGUID: TGUID;
    TempCypherDetails: TFreeOTFECypher;
    TempCypherKey: string;
    TempCypherIV: int64;

    procedure ActivateFreeOTFEComponent();
    procedure DeactivateFreeOTFEComponent();

    procedure InitializeDrives();
    procedure RefreshDrives();
    function  AddIconForDrive(driveLetter: char): integer;

    procedure EnableDisableControls();
    function  GetDriveLetterFromLVItem(listItem: TListItem): char;

    procedure ResizeWindow();

    // Set "mountAsSystem" to either:
    //   FREEOTFE
    //   LINUX
    //   PROMPT
    procedure MountFiles(mountAsSystem: string; filenames: TStringList; readOnly: boolean);

    procedure DriveProperties();

    function  DismountSelected(): boolean;
    function  DismountAll(): boolean;
    function  DismountDrives(dismountDrives: string; isEmergency: boolean): boolean;
    procedure ReportDrivesNotDismounted(drivesRemaining: string; isEmergency: boolean);

    function  PortableModeStart(): boolean;
    function  PortableModeStop(): boolean;

  public
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure WMDeviceChange(var Msg : TMessage); message WM_DEVICECHANGE;

    procedure GenerateOverwriteData(Sender: TObject; passNumber: integer; bytesRequired: cardinal; var outputBlock: TShredBlock);

  end;

var
  frmFreeOTFEMain: TfrmFreeOTFEMain;

implementation

{$R *.DFM}

uses
  ShellApi,  // Required for SHGetFileInfo
  Commctrl,  // Required for ImageList_GetIcon
  ComObj,  // Required for StringToGUID
  Math,  // Requiref for min
  SDUGeneral,
  About_U,
  FreeOTFEVolProperties,
  FreeOTFESelectOverwriteMethod,
  FreeOTFECDBDump,
  FreeOTFECDBBackupRestore,
  OTFEConsts_U,
  SDUFileIterator_U;

const
  CRLF = #13+#10;


  TEXT_MAY_NEED_ADMIN =
           'You may need administrator privileges in order to carry out'+CRLF+
           'this operation.';


  // Format disk related functions ripped from the Unofficial Delphi FAQ (UDF)
  SHFMT_ID_DEFAULT        = $FFFF;
  // Formating options
  SHFMT_OPT_QUICKFORMAT   = $0000;
  SHFMT_OPT_FULL          = $0001;
  SHFMT_OPT_SYSONLY       = $0002;
  // Error codes
  SHFMT_ERROR             = $FFFFFFFF;
  SHFMT_CANCEL            = $FFFFFFFE;
  SHFMT_NOFORMAT          = $FFFFFFFD;

function SHFormatDrive(Handle: HWND; Drive, ID, Options: Word): LongInt;
            stdcall; external 'shell32.dll' name 'SHFormatDrive'


procedure TfrmFreeOTFEMain.RefreshDrives();
var
  ListItem: TListItem;
  driveIconNum: integer;
  i: integer;
  volumeInfo: TOTFEFreeOTFEVolumeInfo;
  mounted: string;
begin
  // Cleardown all...
  lvDrives.Items.Clear();
  ilDriveIcons.Clear();

  if OTFEFreeOTFE.Active then
    begin
    mounted := OTFEFreeOTFE.DrivesMounted();

    for i:=1 to length(mounted) do
      begin
      if OTFEFreeOTFE.GetVolumeInfo(mounted[i], volumeInfo) then
        begin
        ListItem := lvDrives.Items.Add;
        ListItem.Caption := '     '+mounted[i]+':';
        ListItem.SubItems.Add(volumeInfo.Filename);
        driveIconNum := AddIconForDrive(mounted[i]);
        if (driveIconNum>=0) then
          begin
          ListItem.ImageIndex := driveIconNum;
          end;
        end
      else
        begin
        // Strange... The drive probably got dismounted between getting the
        // list of mounted drives, and querying them for more information...
        end;

      end;

    end;

  EnableDisableControls();

end;


// Add an icon to represent the specified drive
// Returns: The icon's index in ilDriveIcons, or -1 on error
function TfrmFreeOTFEMain.AddIconForDrive(driveLetter: char): integer;
var
  iconIdx: integer;
  anIcon: TIcon;
  shfi: SHFileInfo;
  imgListHandle: THandle;
  iconHandle: HICON;
  tmpDrivePath: string;
begin
  iconIdx := -1;
  
  // Create a suitable icon to represent the drive
  tmpDrivePath := driveLetter+':\';
  imgListHandle := SHGetFileInfo(
                                 PChar(tmpDrivePath),
                                 0,
                                 shfi,
                                 sizeof(shfi),
                                 SHGFI_SYSICONINDEX or SHGFI_LARGEICON);
//                                 SHGFI_DISPLAYNAME

  if imgListHandle<>0 then
    begin
    iconHandle := ImageList_GetIcon(imgListHandle, shfi.iIcon, ILD_NORMAL);
    if (iconHandle<>0) then
      begin
      anIcon:= TIcon.Create();
      try
        anIcon.ReleaseHandle();

        anIcon.handle := iconHandle;

        if (ilDriveIcons.Width<anIcon.Width) then
          begin
          ilDriveIcons.Width := anIcon.Width;
          end;
        if (ilDriveIcons.Height<anIcon.Height) then
          begin
          ilDriveIcons.Height := anIcon.Height;
          end;

        iconIdx := ilDriveIcons.addicon(anIcon);

        anIcon.ReleaseHandle();
        DestroyIcon(iconHandle);
      finally
        anIcon.Free();
      end;

      end;
    end;


  Result := iconIdx;

end;

procedure TfrmFreeOTFEMain.InitializeDrives();
var
  tmpColumn: TListColumn;
begin
  // Listview setup...
  lvDrives.SmallImages := ilDriveIcons;
  lvDrives.RowSelect:= TRUE;
  lvDrives.MultiSelect:= TRUE;
  lvDrives.showcolumnheaders := TRUE;
  lvDrives.ReadOnly := TRUE;
  lvDrives.ViewStyle := vsReport;
//    lvDrives.ViewStyle := vsIcon;
//    lvDrives.ViewStyle := vsList;
  lvDrives.IconOptions.Arrangement := iaTop;


  // Listview columns...
  tmpColumn := lvDrives.Columns.Add;
  tmpColumn.Caption := 'Drive';
  tmpColumn.width := 100;

//    tmpColumn.minwidth := lvDrives.width;
  tmpColumn := lvDrives.Columns.Add;
  tmpColumn.Caption := 'Volume';
  tmpColumn.width := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;


// xxx - shouldn't this work?     NewColumn.autosize := TRUE;
// xxx - shouldn't this work?     NewColumn.minwidth := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;


// xxx - use destroyimage or whatever it was to cleardown the **imagelist**

end;


procedure TfrmFreeOTFEMain.miExitClick(Sender: TObject);
begin
  Close();
  
end;

procedure TfrmFreeOTFEMain.FormCreate(Sender: TObject);
begin
  ShuttingDownFlag := FALSE;

  ActivateFreeOTFEComponent();

  InitializeDrives();

  self.Caption := Application.Title;
  
end;

procedure TfrmFreeOTFEMain.miRefreshClick(Sender: TObject);
begin
  RefreshDrives();

end;

procedure TfrmFreeOTFEMain.miNewClick(Sender: TObject);
begin
  if OTFEFreeOTFE.CreateFreeOTFEVolumeWizard() then
    begin
    MessageDlg('FreeOTFE volume created successfully.'+CRLF+
               CRLF+
               'Please mount, format and overwrite this volume''s free space before use.',
               mtInformation, [mbOK], 0);
    end
  else
    begin
    if (OTFEFreeOTFE.LastErrorCode <> OTFE_ERR_USER_CANCEL) then
      begin
      MessageDlg('FreeOTFE volume could not be created', mtError, [mbOK], 0);
      end;
    end;

end;

procedure TfrmFreeOTFEMain.About1Click(Sender: TObject);
var
  aboutDlg: TAbout_F;
begin
  aboutDlg:= TAbout_F.Create(nil);
  try
    aboutDlg.OTFEFreeOTFE := OTFEFreeOTFE;
    aboutDlg.ShowModal();
  finally
    aboutDlg.Free();
  end;

end;

procedure TfrmFreeOTFEMain.lvDrivesResize(Sender: TObject);
begin
  ResizeWindow();

end;


procedure TfrmFreeOTFEMain.FormResize(Sender: TObject);
begin
  ResizeWindow();

end;



// Resize the columns in the drives list so the 2nd column fills out...
procedure TfrmFreeOTFEMain.ResizeWindow();
var
  tmpColumn: TListColumn;
begin
  if lvDrives.columns.count>1 then
    begin
    tmpColumn := lvDrives.columns[1];
    tmpColumn.width := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;
    end;

end;


procedure TfrmFreeOTFEMain.FormShow(Sender: TObject);
begin
  RefreshDrives();
  EnableDisableControls();

end;


procedure TfrmFreeOTFEMain.MountFiles(mountAsSystem: string; filenames: TStringList; readOnly: boolean);
var
  i: integer;
  mountedAs: string;
  msg: string;
  errMsg: string;
  mountedOK: boolean;
begin

  mountAsSystem := uppercase(mountAsSystem);

  if (mountAsSystem = 'FREEOTFE') then
    begin
    mountedOK:= OTFEFreeOTFE.MountFreeOTFE(filenames, mountedAs, readOnly);
    end
  else if (mountAsSystem = 'LINUX') then
    begin
    mountedOK:= OTFEFreeOTFE.MountLinux(filenames, mountedAs, readOnly);
    end
  else
    begin
    mountedOK:= OTFEFreeOTFE.Mount(filenames, mountedAs, readOnly);
    end;

  if not(mountedOK) then
    begin
    if (OTFEFreeOTFE.LastErrorCode <> OTFE_ERR_USER_CANCEL) then
      begin
      errMsg := 'Unable to mount volume; please ensure that you entered the '+CRLF+
                'correct details (password, etc)';
      if (OTFEFreeOTFE.LastErrorCode = OTFE_ERR_NO_FREE_DEVICES) then
        begin
        errMsg := 'Your volume could not be mounted at this time as there are'+CRLF+
                  'no suitable FreeOTFE devices free.'+CRLF+
                  CRLF+
                  'Please dismount one or more volumes, and try again.';
        end;

      MessageDlg(errMsg, mtError, [mbOK], 0);
      end;

    end
  else
    begin
    if (length(mountedAs)=1) then
      begin
      msg := 'Your FreeOTFE volume has been mounted as drive: ';
      end
    else
      begin
      msg := 'Your FreeOTFE volumes have been mounted as drives: ';
      end;

    for i:=1 to length(mountedAs) do
      begin
      if (i<>1) then
        begin
        msg := msg + ', ';
        end;

      msg := msg + mountedAs[i] + ':';
      end;

    RefreshDrives();
    MessageDlg(msg, mtInformation, [mbOK], 0);
    end;

end;


procedure TfrmFreeOTFEMain.miFreeOTFEMountFileClick(Sender: TObject);
begin
  if OTFEFreeOTFE.WarnIfNoHashOrCypherDrivers() then
    begin
    if OpenDialog.Execute() then
      begin
      MountFiles('FREEOTFE', TStringList(OpenDialog.Files), (ofReadOnly in OpenDialog.Options));
      end;
    end;

end;

procedure TfrmFreeOTFEMain.miFreeOTFEMountPartitionClick(Sender: TObject);
var
  selectedPartition: string;
  mountList: TStringList;
begin
  if OTFEFreeOTFE.WarnIfNoHashOrCypherDrivers() then
    begin
    selectedPartition := OTFEFreeOTFE.SelectPartition();

    if (selectedPartition <> '') then
      begin
      mountList:= TStringList.Create();
      try
        mountList.Add(selectedPartition);
        MountFiles('FREEOTFE', mountList, FALSE);
      finally
        mountList.Free();
      end;

      end;
    end;

end;



// Handle files being dropped onto the form from (for example) Windows Explorer.
// Files dropped onto the form should be treated as though the user is trying
// to mount them as volume files
procedure TfrmFreeOTFEMain.WMDropFiles(var Msg: TWMDropFiles);
var
  buffer: array[0..MAX_PATH] of Char;
  numFiles: longint;
  i: integer;
  filesToMount: TStringList;
begin
  try
    // Bring our window to the foreground
    SetForeGroundWindow(self.handle);

    // Handle the dropped files
    numFiles := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0);
    filesToMount := TStringList.Create();
    try
      for i:=0 to (numFiles-1) do
        begin
        DragQueryFile(msg.Drop,
                      i,
                      @buffer,
                      sizeof(buffer));
        filesToMount.Add(buffer);
        end;

      MountFiles('PROMPT', filesToMount, FALSE);

      Msg.Result := 0;
    finally
      filesToMount.Free();
    end;
  finally
    DragFinish(Msg.Drop);
  end;

end;


procedure TfrmFreeOTFEMain.WMDeviceChange(var Msg : TMessage);
begin
  RefreshDrives();

end;


procedure TfrmFreeOTFEMain.DriveProperties();
var
  propertiesDlg: TfrmFreeOTFEVolProperties;
  i: integer;
begin
  for i:=0 to (lvDrives.items.count-1) do
    begin
    if lvDrives.Items[i].Selected then
      begin
      propertiesDlg:= TfrmFreeOTFEVolProperties.Create(nil);
      try
        propertiesDlg.DriveLetter := GetDriveLetterFromLVItem(lvDrives.Items[i]);
        propertiesDlg.OTFEFreeOTFE := OTFEFreeOTFE;
        propertiesDlg.ShowModal();
      finally
        propertiesDlg.Free();
      end;

      end;
    end;

end;


procedure TfrmFreeOTFEMain.miPropertiesClick(Sender: TObject);
begin
  DriveProperties();

end;

procedure TfrmFreeOTFEMain.miPopupPropertiesClick(Sender: TObject);
begin
  DriveProperties();

end;


function TfrmFreeOTFEMain.DismountAll(): boolean;
var
  drivesRemaining: string;
  allOK: boolean;
begin
  allOK := TRUE;

  OTFEFreeOTFE.DismountAll(FALSE);

  drivesRemaining := OTFEFreeOTFE.DrivesMounted();
  if (drivesRemaining<>'') then
    begin
    allOK := FALSE;
    RefreshDrives();
    ReportDrivesNotDismounted(drivesRemaining, FALSE);
    end;

  RefreshDrives();

  Result := allOK;
end;


function TfrmFreeOTFEMain.DismountSelected(): boolean;
var
  i: integer;
  drivesRemaining: string;
  toDismount: string;
  allOK: boolean;
begin
  drivesRemaining := '';

  // First we build up a list of drives to dismount, then we dismount them.
  // This is done since we can't just run through the lvDrives looking for
  // selected items, as they're getting removed while we walk though the list!
  
  for i:=0 to (lvDrives.items.count-1) do
    begin
    if lvDrives.Items[i].Selected then
      begin
      toDismount := toDismount + GetDriveLetterFromLVItem(lvDrives.Items[i]);
      end;
    end;

  allOK := DismountDrives(toDismount, FALSE);

  RefreshDrives();

  Result := allOK;
end;


// Dismount the drives specified
function TfrmFreeOTFEMain.DismountDrives(dismountDrives: string; isEmergency: boolean): boolean;
var
  i: integer;
  tmpDrv: char;
  drivesRemaining: string;
  allOK: boolean;
begin
  allOK := TRUE;
  drivesRemaining := '';

  for i:=1 to length(dismountDrives) do
    begin
    tmpDrv := dismountDrives[i];
    if not(OTFEFreeOTFE.Dismount(tmpDrv, isEmergency)) then
      begin
      drivesRemaining := drivesRemaining + tmpDrv;
      allOK := FALSE;
      end;

    end;

  if (drivesRemaining<>'') then
    begin
    RefreshDrives();
    ReportDrivesNotDismounted(drivesRemaining, isEmergency);
    end;

  RefreshDrives();

  Result := allOK;
end;


// Warn the user that some drives remain mounted, and if the dismount attempted
// wasn't an emergency dismount, then prompt the user if they want to attempt
// an emergency dismount
procedure TfrmFreeOTFEMain.ReportDrivesNotDismounted(drivesRemaining: string; isEmergency: boolean);
var
  msg: string;
  i: integer;
  plural_thisthese: string;
  plural_drivedrives: string;
begin
  plural_thisthese := 'this';
  plural_drivedrives := 'drive';
  if (length(drivesRemaining)>0) then
    begin
    plural_thisthese := 'these';
    plural_drivedrives := 'drives';
    end;

  msg := 'Unable to dismount '+plural_drivedrives+': ';
  for i:=1 to length(drivesRemaining) do
    begin
    if (i<>1) then
      begin
      msg := msg + ', ';
      end;

    msg := msg + drivesRemaining[i] + ':';
    end;


  // If the dismount attempted was a non-emergency dismount; prompt user if
  // they want to attempt an emergency dismount
  if not(isEmergency) then
    begin
    msg := msg+CRLF+
           CRLF+
           'Do you wish to force a dismount on '+plural_thisthese+' '+plural_drivedrives+'?';

    if (MessageDlg(msg, mtConfirmation, [mbYes,mbNo], 0) = mrYes)then
      begin
      if (MessageDlg(
                     'Warning: Emergency dismounts can lead to instability'+CRLF+
                     CRLF+
                     'Are you sure you wish to proceed?',
                     mtWarning,
                     [mbYes,mbNo],
                     0
                    ) = mrYes) then
        begin
        DismountDrives(drivesRemaining, TRUE);
        end;

      end;
    end
  else
    begin
    // Dismount reporting on *was* an emergency dismount; just report remaining
    // drives still mounted
    MessageDlg(msg, mtInformation, [mbOK], 0);
    end;

end;



procedure TfrmFreeOTFEMain.miDismountClick(Sender: TObject);
begin
  DismountSelected();

end;

procedure TfrmFreeOTFEMain.miDismountAllClick(Sender: TObject);
begin
  DismountAll();
  
end;

procedure TfrmFreeOTFEMain.lvDrivesClick(Sender: TObject);
begin
  EnableDisableControls();

end;

procedure TfrmFreeOTFEMain.EnableDisableControls();
var
  drivesSelected: boolean;
  drivesMounted: boolean;
begin
  // The FreeOTFE object may not be active...
  miFreeOTFENew.Enabled            := OTFEFreeOTFE.Active;
  miFreeOTFEMountFile.Enabled      := OTFEFreeOTFE.Active;
  miFreeOTFEMountPartition.Enabled := (OTFEFreeOTFE.Active and OTFEFreeOTFE.CanMountDevice());
  miLinuxVolume.Enabled            := OTFEFreeOTFE.Active;
  miLinuxMountPartition.Enabled    := (OTFEFreeOTFE.Active and OTFEFreeOTFE.CanMountDevice());

  miChangePassword.Enabled := OTFEFreeOTFE.Active;
  miCreateKeyfile.Enabled  := OTFEFreeOTFE.Active;
  miCDB.Enabled            := OTFEFreeOTFE.Active;


  drivesSelected := (lvDrives.selcount > 0);
  drivesMounted  := (lvDrives.Items.count > 0);


  // Menuitems to be enabled/disabled, depending on whether a drive is selected
  miDismountMain.enabled  := drivesSelected;
  miPopupDismount.enabled := drivesSelected;
  miLinuxDismount.enabled := drivesSelected;

  miProperties.enabled      := drivesSelected;
  miPopupProperties.enabled := drivesSelected;

  miFormat.enabled                  := drivesSelected;
  miPopupFormat.enabled             := drivesSelected;
  miPopupOverwriteFreeSpace.enabled := drivesSelected;
  miOverwriteFreeSpace.enabled      := drivesSelected;  // Note: Implies FreeOTFE drivers are running


  // Menuitems to be enabled/disabled, depending on whether one or more drives
  // are mounted
  miDismountAllMain.enabled  := drivesMounted;
  miPopupDismountAll.enabled := drivesMounted;

  // Detect any drivers in portable mode
  miPortableModeDrivers.checked := (OTFEFreeOTFE.DriversInPortableMode() > 0);

end;


function TfrmFreeOTFEMain.GetDriveLetterFromLVItem(listItem: TListItem): char;
var
  tmpDrv: string;
begin
  // Trim the padding whitespace off the drive letter + colon
  tmpDrv := listItem.Caption;
  tmpDrv := TrimLeft(tmpDrv);

  // The first letter of the item's caption is the drive letter
  Result := tmpDrv[1];

end;

procedure TfrmFreeOTFEMain.lvDrivesDblClick(Sender: TObject);
var
  driveLetter: string;
  explorerCommandLine: string;
begin
  if (lvDrives.selcount > 0) then
    begin
    driveLetter := GetDriveLetterFromLVItem(lvDrives.selected);

    if (driveLetter<>'') then
      begin
      explorerCommandLine := 'explorer ' + driveLetter + ':\';

      if (WinExec(PChar(explorerCommandLine), SW_RESTORE))<31 then
        begin
        MessageDlg('Error running explorer', mtError, [], 0);
        end;

      end;
    end;

end;

procedure TfrmFreeOTFEMain.lvDrivesChange(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
  EnableDisableControls();

end;

procedure TfrmFreeOTFEMain.miLinuxMountFileClick(Sender: TObject);
begin
  if OTFEFreeOTFE.WarnIfNoHashOrCypherDrivers() then
    begin
    if OpenDialog.Execute() then
      begin
      MountFiles('LINUX', TStringList(OpenDialog.Files), (ofReadOnly in OpenDialog.Options));
      end;
    end;

end;


procedure TfrmFreeOTFEMain.miLinuxMountPartitionClick(Sender: TObject);
var
  selectedPartition: string;
  mountList: TStringList;
begin
  if OTFEFreeOTFE.WarnIfNoHashOrCypherDrivers() then
    begin
    selectedPartition := OTFEFreeOTFE.SelectPartition();

    if (selectedPartition <> '') then
      begin
      mountList:= TStringList.Create();
      try
        mountList.Add(selectedPartition);
        MountFiles('LINUX', mountList, FALSE);
      finally
        mountList.Free();
      end;

      end;
    end;

end;


procedure TfrmFreeOTFEMain.miLinuxNewClick(Sender: TObject);
begin
  if OTFEFreeOTFE.CreateLinuxVolumeWizard() then
    begin
    MessageDlg('Linux volume created successfully.'+CRLF+
               CRLF+
               'Don''t forget to mount and format this volume before use.',
               mtInformation, [mbOK], 0);
    end
  else
    begin
    if (OTFEFreeOTFE.LastErrorCode <> OTFE_ERR_USER_CANCEL) then
      begin
      MessageDlg('Linux volume could not be created', mtError, [mbOK], 0);
      end;
    end;

end;

procedure TfrmFreeOTFEMain.miFreeOTFEDriversClick(Sender: TObject);
begin
  DeactivateFreeOTFEComponent();
  OTFEFreeOTFE.ShowDriverControlDlg();
  ActivateFreeOTFEComponent();
  EnableDisableControls();

end;




procedure TfrmFreeOTFEMain.ActivateFreeOTFEComponent();
var
  obsoleteDriver: boolean;
begin
  obsoleteDriver:= FALSE;
  try
    OTFEFreeOTFE.Active := TRUE;
  except
    on EFreeOTFEObsoleteDriver do
      begin
      obsoleteDriver := TRUE;
      end;

  else
    // We should only get to here if the FreeOTFE driver isn't both
    // installed & started

  end;


  if (
      (not(OTFEFreeOTFE.Active)) and
      (not(ShuttingDownFlag))  // Don't complain to user is we're about to exit!
     ) then
    begin
    if (obsoleteDriver) then
      begin
      MessageDlg(
                 'In order to use this software, you must first upgrade your '+CRLF+
                 'FreeOTFE drivers to a later version.'+CRLF+
                 CRLF+
                 'If you have just upgraded your copy of FreeOTFE from a '+CRLF+
                 'previous version, please ensure that you have followed the '+CRLF+
                 'upgrade instructions that came with it, and have rebooted '+CRLF+
                 'if necessary.',
                 mtError,
                 [mbOK],
                 0
                );

      end
    else
      begin
      MessageDlg(
                 'Unable to connect to the FreeOTFE driver.'+CRLF+
                 CRLF+
                 'Please ensure that the main "FreeOTFE" driver is installed and started.'+CRLF+
                 CRLF+
                 'If you have only just installed FreeOTFE, please reboot and try again.',
                 mtError,
                 [mbOK],
                 0
                );
      end;

    // Note that we don't terminate here; the user may still use the driver
    // control functionality to install and/or start the FreeOTFE driver
    end;

  // Let Windows know we accept dropped files or not
  DragAcceptFiles(self.Handle, OTFEFreeOTFE.Active);

end;



procedure TfrmFreeOTFEMain.DeactivateFreeOTFEComponent();
begin
  OTFEFreeOTFE.Active := FALSE;

  // Let Windows know we accept dropped files or not
  DragAcceptFiles(self.Handle, OTFEFreeOTFE.Active);

end;




procedure TfrmFreeOTFEMain.miOverwriteFreeSpaceClick(Sender: TObject);
var
  shredder: TShredder;
  i: integer;
  currDrive: char;
  frmOverWriteMethod: TfrmFreeOTFESelectOverwriteMethod;
  overwriteWithEncryptedData: boolean;
  allOK: boolean;
  getRandomBits: integer;
  getRandomBytes: integer;
  randomBuffer: array of byte;
  overwriteOK: integer;
begin
  overwriteOK := 0;
  allOK := FALSE;
  overwriteWithEncryptedData := FALSE;

  frmOverWriteMethod:= TfrmFreeOTFESelectOverwriteMethod.Create(nil);
  try
    frmOverWriteMethod.OTFEFreeOTFEObj := OTFEFreeOTFE;
    if (frmOverWriteMethod.ShowModal() = mrOK) then
      begin
      allOK := TRUE;

      overwriteWithEncryptedData := frmOverWriteMethod.OverwriteWithEncryptedData;
      if (overwriteWithEncryptedData) then
        begin
        TempCypherDriver:= frmOverWriteMethod.CypherDriver;
        TempCypherGUID:= frmOverWriteMethod.CypherGUID;

        OTFEFreeOTFE.GetSpecificCypherDetails(TempCypherDriver, TempCypherGUID, TempCypherDetails);

        // Initilize zeroed IV for encryption
        TempCypherIV := 0;

        // Get *real* random data for encryption key
        getRandomBits := 0;
        if (TempCypherDetails.KeySize < 0) then
          begin
          // -ve keysize = arbitary keysize supported - just use 512 bits
          getRandomBits := 512;
          end
        else if (TempCypherDetails.KeySize > 0) then
          begin
          // +ve keysize = specific keysize - get sufficient bits
          getRandomBits := TempCypherDetails.KeySize;
          end;

        if (getRandomBits>0) then
          begin
          MouseRNGDialog1.RequiredBits := getRandomBits;
          allOK := MouseRNGDialog1.Execute();
          end;

        if (allOK) then
          begin
          getRandomBytes := (getRandomBits div 8);
          SetLength(randomBuffer, getRandomBytes);
          MouseRNGDialog1.RandomData(getRandomBytes, randomBuffer);

          for i:=low(randomBuffer) to high(randomBuffer) do
            begin
            TempCypherKey := TempCypherKey + char(randomBuffer[i]);
            // Overwrite the temp buffer...
            randomBuffer[i] := random(256);
            end;
          SetLength(randomBuffer, 0);
          end;

        end;

      end;

  finally
    frmOverWriteMethod.Free();
  end;



  if (allOK) then
    begin
    if (MessageDlg(
                   'Overwriting all the free space on a large volume may take awhile'+CRLF+
                   CRLF+
                   'Are you sure you wish to proceed?',
                    mtConfirmation,
                   [mbYes,mbNo],
                   0
                  ) = mrYes) then
      begin
      shredder:= TShredder.Create(nil);
      try
        if overwriteWithEncryptedData then
          begin
          shredder.OnOverwriteDataReq := GenerateOverwriteData;
          end
        else
          begin
          shredder.OnOverwriteDataReq := nil;
          end;


        for i:=0 to (lvDrives.items.count-1) do
          begin
          if lvDrives.Items[i].Selected then
            begin
            currDrive := GetDriveLetterFromLVItem(lvDrives.Items[i]);
            overwriteOK := shredder.OverwriteDriveFreeSpace(currDrive, FALSE);
            end;
          end;

        if (overwriteOK = 1) then
          begin
          MessageDlg('Overwrite operation complete.', mtInformation, [mbOK], 0);
          end
        else if (overwriteOK = -1) then
          begin
          MessageDlg(
                     'Overwrite operation FAILED.'+CRLF+
                     CRLF+
                     'Did you remember to format this drive first?',
                     mtError, [mbOK], 0
                    );
          end
        else if (overwriteOK = -2) then
          begin
          MessageDlg('Overwrite operation cancelled by user.', mtInformation, [mbOK], 0);
          end
        else
          begin
          MessageDlg('No drives selected to overwrite?', mtInformation, [mbOK], 0);
          end;


      finally
        shredder.Free();
      end;

    end;  // if (MessageDlg(
  end;  // if (allOK) then

  TempCypherDriver:= '';
  TempCypherGUID:= StringToGUID('{00000000-0000-0000-0000-000000000000}');
  TempCypherKey := '';
  TempCypherIV := 0;

end;


procedure TfrmFreeOTFEMain.miFormatClick(Sender: TObject);
var
  i: integer;
  currDrive: char;
  driveNum: word;
begin
  for i:=0 to (lvDrives.items.count-1) do
    begin
    if lvDrives.Items[i].Selected then
      begin
      currDrive := GetDriveLetterFromLVItem(lvDrives.Items[i]);
      driveNum := ord(currDrive) - ord('A');
      SHFormatDrive(Handle, driveNum, SHFMT_ID_DEFAULT, SHFMT_OPT_FULL);
      end;
    end;

end;


procedure TfrmFreeOTFEMain.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  userConfirm: WORD;
  oldActive: boolean;
begin
  CanClose := TRUE;

  if (CanClose) then
    begin
    oldActive := OTFEFreeOTFE.Active;
    // We (quite reasonably) assume that if the FreeOTFE component is *not*
    // active, and we can't activate it, then no FreeOTFE volumes are mounted
    if not(OTFEFreeOTFE.Active) then
      begin
      // Prevent any warnings...
      ShuttingDownFlag := TRUE;
      ActivateFreeOTFEComponent();
      // Reset flag
      ShuttingDownFlag := FALSE;
      end;

    if (OTFEFreeOTFE.Active) then
      begin
      if (OTFEFreeOTFE.CountDrivesMounted() > 0) then
        begin
        userConfirm := MessageDlg(
                                   'One or volumes are still mounted.'+CRLF+
                                   CRLF+
                                   'Do you wish to dismount all volumes before exiting?',
                                   mtConfirmation, [mbYes, mbNo, mbCancel], 0);

        if (userConfirm = mrCancel) then
          begin
          CanClose := FALSE;
          end
        else if (userConfirm = mrYes) then
          begin
          CanClose := DismountAll();
          end
        else
          begin
          CanClose := TRUE;
          end;

        end;
      end;  // if (OTFEFreeOTFE.Active) then

    if not(oldActive) then
      begin
      // Prevent any warnings...
      ShuttingDownFlag := TRUE;
      DeactivateFreeOTFEComponent();
      // Reset flag
      ShuttingDownFlag := FALSE;
      end;

    end;


  if (CanClose) then
    begin
    if (OTFEFreeOTFE.DriversInPortableMode() > 0) then
      begin
      userConfirm := MessageDlg(
                                 'One or more FreeOTFE drivers is running in portable mode.'+CRLF+
                                 CRLF+
                                 'Do you wish to shutdown portable mode before exiting?',
                                 mtConfirmation, [mbYes, mbNo, mbCancel], 0);

      if (userConfirm = mrCancel) then
        begin
        CanClose := FALSE;
        end
      else if (userConfirm = mrYes) then
        begin
        ShuttingDownFlag := TRUE;
        CanClose := PortableModeStop();
        ShuttingDownFlag := CanClose;
        end
      else
        begin
        CanClose := TRUE;
        end;

      end;

    end;  // if (CanClose) then


  if CanClose then
    begin
    OTFEFreeOTFE.Active := FALSE;
    end;

end;


// The array passed in is zero-indexed; populate elements zero to "bytesRequired"
procedure TfrmFreeOTFEMain.GenerateOverwriteData(Sender: TObject; passNumber: integer; bytesRequired: cardinal; var outputBlock: TShredBlock);
var
  i: integer;
  tempArraySize: cardinal;
  blocksizeBytes: cardinal;
  plaintext: string;
  cyphertext: string;
  IV: string;
  localIV: int64;
begin
  // Generate an array of random data containing "bytesRequired" bytes of data,
  // plus additional random data to pad out to the nearest multiple of the
  // cypher's blocksize bits
  // Cater for if the blocksize was -ve or zero
  if (TempCypherDetails.BlockSize < 1) then
    begin
    blocksizeBytes := 1;
    end
  else
    begin
    blocksizeBytes := (TempCypherDetails.BlockSize div 8);
    end;
  tempArraySize := bytesRequired + (blocksizeBytes - (bytesRequired mod blocksizeBytes));

  plaintext := '';
  for i:=1 to tempArraySize do
    begin
    plaintext := plaintext + char(random(256));
    end;


  // Adjust the IV so that this block of encrypted pseudorandom data should be
  // reasonably unique
  IV := '';
  if (TempCypherDetails.BlockSize > 0) then
    begin
    IV := StringOfChar(#0, (TempCypherDetails.BlockSize div 8));

    inc(TempCypherIV);
    localIV := TempCypherIV;
    
    for i:=1 to min(sizeof(localIV), length(IV)) do
      begin
      IV[i] := char((localIV AND $FF));
      localIV := localIV shr 8;
      end;

    end;

  // Encrypt the pseudorandom data generated
  if not(OTFEFreeOTFE.EncryptData(
                                  TempCypherDriver,
                                  TempCypherGUID,
                                  TempCypherKey,
                                  IV,
                                  plaintext,
                                  cyphertext
                                 )) then
    begin
    MessageDlg(
               'Unable to encrypt pseudorandom data before using for '+CRLF+
               'overwrite buffer?!',
               mtWarning,
               [mbOK],
               0
              );
    end;

  // Copy the encrypted data into the outputBlock
  for i:=0 to (bytesRequired-1) do
    begin
    outputBlock[i] := byte(cyphertext[i+1]);
    end;

end;



function TfrmFreeOTFEMain.PortableModeStart(): boolean;
var
  driverFilenames: TStringList;
  fileIterator: TSDUFileIterator;
  filename: string;
  allOK: boolean;
begin
  allOK:= FALSE;

  driverFilenames:= TStringList.Create();
  try
    // Compile a list of drivers in the CWD
    fileIterator:= TSDUFileIterator.Create(nil);
    try
      fileIterator.Directory := ExtractFilePath(Application.ExeName);
      fileIterator.FileMask := '*.sys';
      fileIterator.RecurseSubDirs := FALSE;
      fileIterator.OmitStartDirPrefix := FALSE;
      fileIterator.IncludeDirNames := FALSE;

      fileIterator.Reset();
      filename := fileIterator.Next();
      while (filename<>'') do
        begin
        driverFilenames.Add(filename);
        filename := fileIterator.Next();
        end;

    finally
      fileIterator.Free();
    end;


    if (driverFilenames.count<1) then
      begin
      MessageDlg(
                 'Unable to locate any portable FreeOTFE drivers.'+CRLF+
                 CRLF+
                 'Please ensure that the a copy of the FreeOTFE drivers (".sys" files) '+CRLF+
                 'you wish to use are located in the correct directory.',
                 mtWarning,
                 [mbOK],
                 0
                );
      end
    else
      begin
      DeactivateFreeOTFEComponent();
      if OTFEFreeOTFE.PortableStart(driverFilenames) then
        begin
        MessageDlg('Portable mode drivers installed and started.', mtInformation, [mbOK], 0);
        allOK:= TRUE;
        end
      else
        begin
        MessageDlg(
                   'One or more of your portable FreeOTFE drivers could not be '+CRLF+
                   'installed/started.'+CRLF+
                   CRLF+
                   TEXT_MAY_NEED_ADMIN+CRLF+
                   CRLF+
                   'Please select "File | Drivers..." to check which drivers are '+CRLF+
                   'currently operating.',
                   mtWarning,
                   [mbOK],
                   0
                  );
        end;

      end;

  finally
    driverFilenames.Free();
    ActivateFreeOTFEComponent();
  end;

  EnableDisableControls();

  Result := allOK;
end;

function TfrmFreeOTFEMain.PortableModeStop(): boolean;
var
  allOK: boolean;
  stopOK: boolean;
begin
  allOK:= FALSE;

  // Note that if the component was *not* active, then we can shutdown all
  // portable mode drivers; if we aren't active, this implies the main driver
  // isn't installed/running - in which case we don't need the user to confirm
  // as no drives can be mounted
  stopOK := TRUE;
  if OTFEFreeOTFE.Active then
    begin
    if (OTFEFreeOTFE.CountDrivesMounted() > 0) then
      begin
      stopOK := (MessageDlg(
                             'You have one or more volumes mounted.'+CRLF+
                             CRLF+
                             'If any of the currently mounted volumes makes use of any of the '+CRLF+
                             'FreeOTFE drivers which are currently in portable mode, '+CRLF+
                             'stopping portable mode can cause SYSTEM INSTABILITY.'+CRLF+
                             CRLF+
                             'It is recommended that you dismount all volumes before '+CRLF+
                             'stopping portable mode.'+CRLF+
                             CRLF+
                             'Do you wish to continue stopping portable mode?',
                             mtWarning,
                             [mbYes,mbNo],
                             0
                            ) = mrYes);
      end;
    end;
    

  if stopOK then
    begin
    DeactivateFreeOTFEComponent();
    if OTFEFreeOTFE.PortableStop() then
      begin
      // No point in informing user; they would have been prompted if they
      // wanted to do this, and they can tell they've stopped as the
      // application will just exit; only popping up a messagebox if there's a
      // problem
      if not(ShuttingDownFlag) then
        begin
        MessageDlg('Portable mode drivers stopped and uninstalled.', mtInformation, [mbOK], 0);
        end;

      allOK := TRUE;
      end
    else
      begin
      MessageDlg(
                 'One or more of your portable FreeOTFE drivers could not be '+CRLF+
                 'stopped/uninstalled.'+CRLF+
                 CRLF+
                 TEXT_MAY_NEED_ADMIN+CRLF+
                 CRLF+
                 'Please select "File | Drivers..." to check which drivers are '+CRLF+
                 'currently operating.',
                 mtWarning,
                 [mbOK],
                 0
                );
      end;

    ActivateFreeOTFEComponent();
    EnableDisableControls();
    end;
     

  Result := allOK;
end;

procedure TfrmFreeOTFEMain.miPortableModeDriversClick(Sender: TObject);
begin
  if miPortableModeDrivers.checked then
    begin
    PortableModeStop();
    end
  else
    begin
    PortableModeStart();
    end;

  miPortableModeDrivers.checked := (OTFEFreeOTFE.DriversInPortableMode() > 0);
  EnableDisableControls();

end;


procedure TfrmFreeOTFEMain.miCDBBackupClick(Sender: TObject);
var
  dlg: TfrmFreeOTFECDBBackupRestore;
begin
  dlg:= TfrmFreeOTFECDBBackupRestore.Create(nil);
  try
    dlg.OTFEFreeOTFE := OTFEFreeOTFE;
    dlg.DlgType := opBackup;
    dlg.ShowModal();
  finally
    dlg.Free();
  end;

end;

procedure TfrmFreeOTFEMain.miCDBRestoreClick(Sender: TObject);
var
  dlg: TfrmFreeOTFECDBBackupRestore;
begin
  dlg:= TfrmFreeOTFECDBBackupRestore.Create(nil);
  try
    dlg.OTFEFreeOTFE := OTFEFreeOTFE;
    dlg.DlgType := opRestore;
    dlg.ShowModal();
  finally
    dlg.Free();
  end;

end;


procedure TfrmFreeOTFEMain.miCreateKeyfileClick(Sender: TObject);
begin
  OTFEFreeOTFE.WizardCreateKeyfile();
end;

procedure TfrmFreeOTFEMain.miChangePasswordClick(
  Sender: TObject);
begin
  OTFEFreeOTFE.WizardChangePassword();
end;

procedure TfrmFreeOTFEMain.miCDBPlaintextDumpClick(Sender: TObject);
var
  dlg: TfrmFreeOTFECDBDump;
begin
  if OTFEFreeOTFE.WarnIfNoHashOrCypherDrivers() then
    begin
    dlg:= TfrmFreeOTFECDBDump.Create(nil);
    try
      dlg.OTFEFreeOTFE := OTFEFreeOTFE;
      dlg.ShowModal();
    finally
      dlg.Free();
    end;

    end;

end;


END.



